@@ -18,10 +18,15 @@ if not PyNormaliz.NmzHasEAntic():
     skip.add("segfault-27.txt")
     skip.add("segfault-35.txt")
     skip.add("vertex_denom-37.txt")
+    skip.add("modify_cone_renf.txt")
+    
+if not PyNormaliz.NmzHasCocoa():
+    skip.add("test_rational_cones.txt")
 
 # run doctests
 attempted = failed = 0
-for filename in os.listdir(os.curdir):
+dir = os.path.dirname(os.path.realpath(__file__))
+for filename in os.listdir(dir):
     if filename.endswith('.txt'):
         if filename in skip:
             print("Skip {}".format(filename))
@@ -40,8 +45,9 @@ for filename in os.listdir(os.curdir):
 
 
 # unit tests
-import generic_test
+from generic_test import tests
 
-generic_test.Test1(verbose=False).run(repeat=100)
+for test in tests:
+    failed += test(verbosity=2).run(repeat=10)
 
 sys.exit(failed)
