@@ -2,21 +2,27 @@
 
 set -e
 
-echo "::group:flint"
+echo "::group::flint"
 
 source $(dirname "$0")/common.sh
 
-CONFIGURE_FLAGS="--prefix=${PREFIX} --with-mpfr=${PREFIX} ${EXTRA_FLINT_FLAGS}"
+CONFIGURE_FLAGS="--prefix=${PREFIX}"
+
+if [ "$OSTYPE" != "msys" ]; then
+	CONFIGURE_FLAGS="${CONFIGURE_FLAGS} --with-mpfr=${PREFIX}"
+else # only static here, we take shared from MSYS repository
+	CONFIGURE_FLAGS="${CONFIGURE_FLAGS} --with-gmp=${MSYS_STANDARD_LOC} --with-mpfr=${MSYS_STANDARD_LOC} --disable-shared"
+fi
+
 if [ "$GMP_INSTALLDIR" != "" ]; then
     CONFIGURE_FLAGS="${CONFIGURE_FLAGS} --with-gmp=${GMP_INSTALLDIR}"
 fi
 
 ## script for the installation of Flint for the use in libnormaliz
 
-FLINT_VERSION="2.6.3"
-FLINT_URL="http://www.flintlib.org/flint-${FLINT_VERSION}.tar.gz"
-FLINT_SHA256=ce1a750a01fa53715cad934856d4b2ed76f1d1520bac0527ace7d5b53e342ee3
-
+FLINT_VERSION="2.8.4"
+FLINT_URL="https://flintlib.org/flint-${FLINT_VERSION}.tar.gz"
+FLINT_SHA256=61df92ea8c8e9dc692d46c71d7f50aaa09a33d4ba08d02a1784730a445e5e4be
 
 echo "Installing FLINT..."
 
@@ -35,4 +41,4 @@ if [[ $OSTYPE == "linux-gnu" ]]; then
     sed -i s/"-Wl,"// Makefile.subdirs
 fi
 # make -j4 # verbose
-make install -j4
+make install -j8
