@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -26,30 +26,34 @@
 #include "libnormaliz/vector_operations.h"
 
 namespace libnormaliz {
-    
+
 using namespace std;
-    
+
 template <typename Integer>
 FaceLattice<Integer>::FaceLattice() {
-    
 }
 
 // It is assumed that the matrices in the constructor are for the pointed quotient,
 // even if the names of the parameters don't indicate that.
-    
+
 template <typename Integer>
-FaceLattice<Integer>::FaceLattice(const Matrix<Integer>& SupportHyperplanes, const Matrix<Integer>& VerticesOfPolyhedron, 
-        const Matrix<Integer>& ExtremeRaysRecCone, const bool cone_inhomogeneous){
-    
+FaceLattice<Integer>::FaceLattice(Matrix<Integer>& SupportHyperplanes,
+                                  const Matrix<Integer>& VerticesOfPolyhedron,
+                                  const Matrix<Integer>& ExtremeRaysRecCone,
+                                  const bool cone_inhomogeneous,
+                                  bool swap_allowed) {
     inhomogeneous = cone_inhomogeneous;
-    
+
     nr_supphyps = SupportHyperplanes.nr_of_rows();
     nr_extr_rec_cone = ExtremeRaysRecCone.nr_of_rows();
     nr_vert = VerticesOfPolyhedron.nr_of_rows();
     nr_gens = nr_extr_rec_cone + nr_vert;
-    
-    SuppHyps = SupportHyperplanes; 
-    dim = SupportHyperplanes[0].size();
+
+    if (swap_allowed)
+        swap(SuppHyps, SupportHyperplanes);
+    else
+        SuppHyps = SupportHyperplanes;
+    dim = SuppHyps[0].size();
 
     SuppHypInd.clear();
     SuppHypInd.resize(nr_supphyps);
@@ -79,7 +83,7 @@ FaceLattice<Integer>::FaceLattice(const Matrix<Integer>& SupportHyperplanes, con
 
             if (inhomogeneous) {
                 for (size_t j = 0; j < nr_vert; ++j) {
-                    if (v_scalar_product(SupportHyperplanes[i], VerticesOfPolyhedron[j]) == 0) {
+                    if (v_scalar_product(SuppHyps[i], VerticesOfPolyhedron[j]) == 0) {
                         nr_gens_in_hyp++;
                         SuppHypInd[i][j] = true;
                     }
@@ -87,14 +91,14 @@ FaceLattice<Integer>::FaceLattice(const Matrix<Integer>& SupportHyperplanes, con
             }
 
             for (size_t j = 0; j < nr_extr_rec_cone; ++j) {
-                if (v_scalar_product(SupportHyperplanes[i], ExtremeRaysRecCone[j]) == 0) {
+                if (v_scalar_product(SuppHyps[i], ExtremeRaysRecCone[j]) == 0) {
                     nr_gens_in_hyp++;
                     SuppHypInd[i][j + nr_vert] = true;
                 }
             }
 
             if (nr_gens_in_hyp == (int)(dim - 1))
-//#pragma omp atomic
+                //#pragma omp atomic
                 nr_simplial_facets++;
 
         } catch (const std::exception&) {
@@ -108,7 +112,6 @@ FaceLattice<Integer>::FaceLattice(const Matrix<Integer>& SupportHyperplanes, con
 
     // if (verbose)
     //    verboseOutput() << "Simplicial facets " << nr_simplial_facets << " of " << nr_supphyps << endl;
-    
 }
 
 struct FaceInfo {
@@ -126,10 +129,9 @@ bool face_compare(const pair<dynamic_bitset, FaceInfo>& a, const pair<dynamic_bi
 
 template <typename Integer>
 void FaceLattice<Integer>::compute(const long face_codim_bound, const bool verbose, bool change_integer_type) {
-
     bool bound_codim = false;
     if (face_codim_bound >= 0)
-        bound_codim = true;    
+        bound_codim = true;
 
     dynamic_bitset SimpleVert(nr_gens);
     size_t nr_simpl = 0;
@@ -168,7 +170,7 @@ void FaceLattice<Integer>::compute(const long face_codim_bound, const bool verbo
     }
 
     Matrix<MachineInteger> SuppHyps_MI;
-    if(change_integer_type)
+    if (change_integer_type)
         convert(SuppHyps_MI, SuppHyps);
 
     /*for(int i=0;i< 10000;++i){ // for pertubation of order of supphyps
@@ -215,7 +217,7 @@ void FaceLattice<Integer>::compute(const long face_codim_bound, const bool verbo
         }
 
         long step_x_size = nr_faces - VERBOSE_STEPS;
-        
+
         bool skip_remaining = false;
         std::exception_ptr tmp_exception;
 
@@ -483,24 +485,21 @@ void FaceLattice<Integer>::compute(const long face_codim_bound, const bool verbo
         verboseOutput() << endl << "Total number of faces computed " << total_nr_faces << endl;
         verboseOutput() << "f-vector " << f_vector;
     }
-
-    
 }
 
 template <typename Integer>
-vector<size_t> FaceLattice<Integer>::getFVector(){
-    return f_vector;    
+vector<size_t> FaceLattice<Integer>::getFVector() {
+    return f_vector;
 }
 
 template <typename Integer>
-void FaceLattice<Integer>::get(map<dynamic_bitset, int>& FaceLatticeOutput){
-    swap(FaceLat,FaceLatticeOutput);
-    
+void FaceLattice<Integer>::get(map<dynamic_bitset, int>& FaceLatticeOutput) {
+    swap(FaceLat, FaceLatticeOutput);
 }
 
 template <typename Integer>
-void FaceLattice<Integer>::get(vector<dynamic_bitset>& SuppHypIndOutput){
-    swap(SuppHypInd,SuppHypIndOutput);
+void FaceLattice<Integer>::get(vector<dynamic_bitset>& SuppHypIndOutput) {
+    swap(SuppHypInd, SuppHypIndOutput);
 }
 
 #ifndef NMZ_MIC_OFFLOAD  // offload with long is not supported
