@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -25,10 +25,150 @@
 #include <cctype>  // std::isdigit
 #include <limits>  // numeric_limits
 
+// #include <boost/smart_ptr/intrusive_ptr.hpp>
+
 #include "libnormaliz/input.h"
+#include "libnormaliz/list_and_map_operations.h"
 
 namespace libnormaliz {
 
+//---------------------------------------------------------------------------
+//                     Number input
+//---------------------------------------------------------------------------
+
+
+#ifdef ENFNORMALIZ
+
+static int xalloc = std::ios_base::xalloc();
+
+// Normaliz implementation of deprecated e-antic functions
+
+std::istream & nmz_set_pword(boost::intrusive_ptr<const renf_class> our_renf, std::istream & is)
+{
+    is.pword(xalloc) = const_cast<void*>(reinterpret_cast<const void*>(&*our_renf));
+    return is;
+}
+
+
+boost::intrusive_ptr<const renf_class> nmz_get_pword(std::istream& is) {
+    return reinterpret_cast<renf_class*>(is.pword(xalloc));
+}
+#endif
+
+
+// To be used in input.cpp
+inline void string2coeff(mpq_class& coeff, istream& in, const string& s) {  // in here superfluous parameter
+
+    stringstream sin(s);
+    coeff = mpq_read(sin);
+    // coeff=mpq_class(s);
+}
+
+inline void read_number(istream& in, mpq_class& number) {
+    number = mpq_read(in);
+}
+
+inline void read_number(istream& in, long& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, long long& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, nmz_float& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, mpz_class& number) {
+    in >> number;
+}
+
+#ifdef ENFNORMALIZ
+
+inline void string2coeff(renf_elem_class& coeff, istream& in, const string& s) {  // we need in to access the renf
+
+    try {
+        coeff = renf_elem_class(*nmz_get_pword(in), s);
+        // coeff = renf_elem_class(*renf_class::get_pword(in), s);
+    } catch (const std::exception& e) {
+        cerr << e.what() << endl;
+        throw BadInputException("Illegal number string " + s + " in input, Exiting.");
+    }
+}
+
+inline void read_number(istream& in, renf_elem_class& number) {
+    // in >> number;
+
+    char c;
+
+    in >> ws;
+    c = in.peek();
+    if (c != '(' && c != '\'' && c != '\"') {  // rational number
+        mpq_class rat = mpq_read(in);
+        number = renf_elem_class(rat);
+        return;
+    }
+
+    // now we have a proper field element
+
+    in >> c;  // read (
+
+    string num_string;
+    bool skip = false;
+    while (in.good()) {
+        c = in.peek();
+        if (c == ')' || c == '\'' || c == '\"') {
+            in >> c;
+            break;
+        }
+        if (c == '~' || c == '=' || c == '[')  // skip the approximation
+            skip = true;
+        in.get(c);
+        if (in.fail())
+            throw BadInputException("Error in reading number: field element not terminated");
+        if (!skip)
+            num_string += c;
+    }
+    string2coeff(number, in, num_string);
+}
+#endif
+
+// matrix input
+
+template <typename Integer>
+Matrix<Integer> readMatrix(const string project) {
+    // reads one matrix from file with name project
+    // format: nr of rows, nr of colimns, entries
+    // all separated by white space
+
+    string name_in = project;
+    const char* file_in = name_in.c_str();
+    ifstream in;
+    in.open(file_in, ifstream::in);
+    if (in.is_open() == false)
+        throw BadInputException("readMatrix cannot find file");
+    int nrows, ncols;
+    in >> nrows;
+    in >> ncols;
+
+    if (nrows == 0 || ncols == 0)
+        throw BadInputException("readMatrix finds matrix empty");
+
+    int i, j;
+    Matrix<Integer> result(nrows, ncols);
+
+    for (i = 0; i < nrows; ++i)
+        for (j = 0; j < ncols; ++j) {
+            read_number(in, result[i][j]);
+            if (in.fail())
+                throw BadInputException("readMatrix finds matrix corrupted");
+        }
+    return result;
+}
+
+//--------------------------------------------------------------------------
+
 // eats up a comment, stream must start with "/*", eats everything until "*/"
 void skip_comment(istream& in) {
     int i = in.get();
@@ -46,41 +186,42 @@ void skip_comment(istream& in) {
 }
 
 template <typename Number>
-void save_matrix(map<Type::InputType, vector<vector<Number> > >& input_map,
+void save_matrix(map<Type::InputType, Matrix<Number> >& input_map,
                  InputType input_type,
-                 const vector<vector<Number> >& M) {
+                 const Matrix<Number>& M) {
     // check if this type already exists
     if (contains(input_map, input_type)) {
         /*throw BadInputException("Multiple inputs of type \"" + type_string
                 + "\" are not allowed!");*/
-        input_map[input_type].insert(input_map[input_type].end(), M.begin(), M.end());
+        input_map[input_type].append(M);
         return;
     }
     input_map[input_type] = M;
 }
 
 template <typename Number>
-void save_empty_matrix(map<Type::InputType, vector<vector<Number> > >& input_map, InputType input_type) {
-    vector<vector<Number> > M;
+void save_empty_matrix(map<Type::InputType, Matrix<Number> >& input_map, InputType input_type) {
+    Matrix<Number> M;
     save_matrix(input_map, input_type, M);
 }
-
+ /*
 template <typename Number>
-vector<vector<Number> > transpose_mat(const vector<vector<Number> >& mat) {
-    if (mat.size() == 0 || mat[0].size() == 0)
-        return vector<vector<Number> >(0);
+Matrix<Number> transpose_mat(const Matrix<Number>& mat) {
+    if (mat.nr_of_rows() == 0 || mat[0].size() == 0)
+        return Matrix<Number>(0);
     size_t m = mat[0].size();
-    size_t n = mat.size();
-    vector<vector<Number> > transpose(m, vector<Number>(n, 0));
+    size_t n = mat.nr_of_rows()
+    Matrix<Number> transpose(m, vector<Number>(n, 0));
     for (size_t i = 0; i < m; ++i)
         for (size_t j = 0; j < n; ++j)
             transpose[i][j] = mat[j][i];
     return transpose;
 }
+*/
 
 template <typename Number>
-void append_row(const vector<Number> row, map<Type::InputType, vector<vector<Number> > >& input_map, Type::InputType input_type) {
-    vector<vector<Number> > one_row(1, row);
+void append_row(const vector<Number> row, map<Type::InputType, Matrix<Number> >& input_map, Type::InputType input_type) {
+    Matrix<Number> one_row(row);
     save_matrix(input_map, input_type, one_row);
 }
 
@@ -89,7 +230,7 @@ void process_constraint(const string& rel,
                         const vector<Number>& left,
                         Number right,
                         const Number modulus,
-                        map<Type::InputType, vector<vector<Number> > >& input_map,
+                        map<Type::InputType, Matrix<Number> >& input_map,
                         bool forced_hom) {
     vector<Number> row = left;
     bool inhomogeneous = false;
@@ -110,6 +251,9 @@ void process_constraint(const string& rel,
     if (strict_inequality && forced_hom) {
         throw BadInputException("Strict inequality not allowed in hom_constraints!");
     }
+    if (strict_inequality && using_renf<Number>()) {
+        throw BadInputException("Strict inequality not allowed for algebraic polyhedra!");
+    }
     if (inhomogeneous || forced_hom)
         row.push_back(-right);   // rhs --> lhs
     if (modified_rel == "<=") {  // convert <= to >=
@@ -117,8 +261,11 @@ void process_constraint(const string& rel,
             row[j] = -row[j];
         modified_rel = ">=";
     }
-    if (rel == "~")
+    if (rel == "~") {
+        if (using_renf<Number>())
+            throw BadInputException("Congruence not allowed for algebraic polyhedra");
         row.push_back(modulus);
+    }
 
     if (inhomogeneous && !forced_hom) {
         if (modified_rel == "=") {
@@ -153,6 +300,8 @@ void process_constraint(const string& rel,
 
 template <typename Number>
 bool read_modulus(istream& in, Number& modulus) {
+    if (using_renf<Number>())
+        throw BadInputException("Congruence not allowed for field coefficients");
     in >> std::ws;  // gobble any leading white space
     char dummy;
     in >> dummy;
@@ -168,6 +317,17 @@ bool read_modulus(istream& in, Number& modulus) {
     return true;
 }
 
+void check_modulus(mpq_class& modulus) {
+    if (modulus <= 0 || modulus.get_den() != 1)
+        throw BadInputException("Error in modulus of congruence");
+}
+
+#ifdef ENFNORMALIZ
+void check_modulus(renf_elem_class& modulus) {
+    throw BadInputException("Congruences not allowed for algebraic polyhedra");
+}
+#endif
+
 template <typename Number>
 void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Number& right, Number& modulus, bool forced_hom) {
     string constraint;
@@ -287,6 +447,8 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
 
     // now we split off the modulus if necessary
     if (rel == "~") {
+        if (using_renf<Number>())
+            throw BadInputException("Congruence not allowed for algebraic polyhedra");
         string last_term = terms.back();
         size_t last_bracket_at = 0;
         bool has_bracket = false;
@@ -305,8 +467,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         modulus = mpq_class(modulus_string);
         // modulus.canonicalize();
         // cout << "mod " << modulus << endl;
-        if (modulus <= 0 || modulus.get_den() != 1)
-            throw BadInputException("Error in modulus of congruence");
+        check_modulus(modulus);
     }
 
     // for(size_t i=0;i<terms.size();++i)
@@ -400,7 +561,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
 }
 
 template <typename Number>
-void read_constraints(istream& in, long dim, map<Type::InputType, vector<vector<Number> > >& input_map, bool forced_hom) {
+void read_constraints(istream& in, long dim, map<Type::InputType, Matrix<Number> >& input_map, bool forced_hom) {
     long nr_constraints;
     in >> nr_constraints;
 
@@ -443,7 +604,6 @@ void read_constraints(istream& in, long dim, map<Type::InputType, vector<vector<
             read_number(in, right);
             if (rel == "~") {
                 if (!read_modulus(in, modulus))
-                    // throw BadInputException("Congruence not allowed with field coefficients!");
                     throw BadInputException("Error while reading modulus!");
             }
             if (in.fail()) {
@@ -461,28 +621,48 @@ bool read_sparse_vector(istream& in, vector<Number>& input_vec, long length) {
 
     while (in.good()) {
         in >> std::ws;
-        int c = in.peek();
+        char c = in.peek();
         if (c == ';') {
             in >> dummy;  // swallow ;
             return true;
         }
-        long pos;
-        in >> pos;
-        if (in.fail())
+        string range;
+        while (true) {
+            in >> c;
+            if (in.fail())
+                return false;
+            if (c != ':')
+                range += c;
+            else
+                break;
+        }
+        int first_pos = -1, last_pos = -1;
+        size_t found_dots = range.find("..", 0);
+        if (found_dots != string::npos) {
+            if (found_dots == 0)
+                return false;
+            first_pos = stoi(range.substr(0, found_dots));
+            first_pos--;
+            last_pos = stoi(range.substr(found_dots + 2));
+            last_pos--;
+        }
+        else {
+            first_pos = stoi(range);
+            first_pos--;
+            last_pos = first_pos;
+        }
+
+        if (first_pos < 0 || first_pos >= length)
             return false;
-        pos--;
-        if (pos < 0 || pos >= length)
+        if (last_pos < first_pos || last_pos >= length)
             return false;
-        in >> std::ws;
-        c = in.peek();
-        if (c != ':')
-            return false;
-        in >> dummy;  // skip :
+
         Number value;
         read_number(in, value);
         if (in.fail())
             return false;
-        input_vec[pos] = value;
+        for (int i = first_pos; i <= last_pos; ++i)
+            input_vec[i] = value;
     }
 
     return false;
@@ -534,7 +714,8 @@ void read_polynomial(istream& in, string& polynomial) {
 }
 
 template <typename Number>
-bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool transpose) {
+bool read_formatted_matrix(istream& in, Matrix<Number>& input_Matrix, bool transpose) {
+    vector<vector<Number> > input_mat;
     input_mat.clear();
     in >> std::ws;
     char dummy;
@@ -546,8 +727,9 @@ bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool
         in >> std::ws;
         if (!one_more_entry_required && in.peek() == ']') {  // closing ] found
             in >> dummy;
+            input_Matrix = Matrix<Number>(input_mat);
             if (transpose)
-                input_mat = transpose_mat(input_mat);
+                input_Matrix = input_Matrix.transpose();
             return true;
         }
         vector<Number> input_vec;
@@ -569,14 +751,7 @@ bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool
     return false;
 }
 
-template <typename Number>
-void read_number_field(istream& in, renf_class& number_field) {
-    throw NumberFieldInputException();
-}
-
-#ifdef ENFNORMALIZ
-template <>
-void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
+void read_number_field_strings(istream& in, string& mp_string, string& indet, string& emb_string) {
     char c;
     string s;
     in >> s;
@@ -588,7 +763,6 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
         throw BadInputException("Error in reading number field: min_poly does not start with (");
     in >> c;
 
-    string mp_string;
     while (in.good()) {
         c = in.peek();
         if (c == ')') {
@@ -601,24 +775,21 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
         mp_string += c;
     }
     // omp_set_num_threads(1);
-    
-    string indet;
-    
-    for(auto& g:mp_string){
-        if(isalpha(g)){
+
+    for (auto& g : mp_string) {
+        if (isalpha(g)) {
             indet = g;
             break;
         }
     }
-    
-    if(indet == "e" || indet == "x")
+
+    if (indet == "e" || indet == "x")
         throw BadInputException("Letters e and x not allowed for field generator");
 
     in >> s;
     if (s != "embedding")
         throw BadInputException("Error in reading number field: expected keyword embedding");
     in >> ws;
-    string emb_string;
     c = in.peek();
     if (c == '[') {
         in >> c;
@@ -637,11 +808,18 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
 
     if (in.fail())
         throw BadInputException("Could not read number field!");
+}
 
-    renf = renf_class(mp_string, indet, emb_string);
-    renf.gen_name = indet; // temporary fix for bug in renfxx.h
-    
-    renf.set_istream(in); 
+#ifdef ENFNORMALIZ
+renf_class_shared read_number_field(istream& in) {
+    string mp_string, indet, emb_string;
+    read_number_field_strings(in, mp_string, indet, emb_string);
+
+    auto renf = renf_class::make(mp_string, indet, emb_string);
+    // renf->set_pword(in);
+    nmz_set_pword(renf,in);
+
+    return renf;
 }
 #endif
 
@@ -654,11 +832,11 @@ void read_num_param(istream& in, map<NumParam::Param, long>& num_param_input, Nu
 }
 
 template <typename Number>
-map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
+InputMap<Number> readNormalizInput(istream& in,
                                                                  OptionsHandler& options,
                                                                  map<NumParam::Param, long>& num_param_input,
                                                                  string& polynomial,
-                                                                 renf_class& number_field) {
+                                                                 renf_class_shared& number_field) {
     string type_string;
     long i, j;
     long nr_rows, nr_columns, nr_rows_or_columns;
@@ -669,7 +847,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
     set<NumParam::Param> num_par_already_set;
     bool we_have_a_polynomial = false;
 
-    map<Type::InputType, vector<vector<Number> > > input_map;
+    InputMap<Number> input_map;
 
     in >> std::ws;  // eat up any leading white spaces
     int c = in.peek();
@@ -743,6 +921,10 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     options.activateNoExtRaysOutput();
                     continue;
                 }
+                if (type_string == "NoHilbertBasisOutput") {
+                    options.activateNoHilbertBasisOutput();
+                    continue;
+                }
                 if (type_string == "NoMatricesOutput") {
                     options.activateNoMatricesOutput();
                     continue;
@@ -755,7 +937,11 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
 #ifndef ENFNORMALIZ
                     throw BadInputException("number_field only allowed for Normaliz with e-antic");
 #else
-                    read_number_field<Number>(in, number_field);
+                    if (!std::is_same<Number, renf_elem_class>::value) {
+                        throw NumberFieldInputException();
+                    }
+                    // TODO: Err if Number is not renf_elem_class
+                    number_field = read_number_field(in);
 #endif
                     continue;
                 }
@@ -764,8 +950,9 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         throw BadInputException("Ambient space must be known for " + type_string + "!");
                     }
                     input_type = Type::grading;
-                    save_matrix(input_map, input_type,
-                                vector<vector<Number> >(1, vector<Number>(dim + type_nr_columns_correction(input_type), 1)));
+                    vector<Number> TotDeg(dim + type_nr_columns_correction(input_type),1);
+                    Matrix<Number> TotDegMat(TotDeg);
+                    save_matrix(input_map, input_type, TotDegMat);
                     continue;
                 }
                 if (type_string == "nonnegative") {
@@ -773,8 +960,8 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         throw BadInputException("Ambient space must be known for " + type_string + "!");
                     }
                     input_type = Type::signs;
-                    save_matrix(input_map, input_type,
-                                vector<vector<Number> >(1, vector<Number>(dim + type_nr_columns_correction(input_type), 1)));
+                    Matrix<Number> SignMat(vector<Number>(dim + type_nr_columns_correction(input_type), 1));
+                    save_matrix(input_map, input_type, SignMat);
                     continue;
                 }
                 if (type_string == "constraints") {
@@ -824,7 +1011,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                             throw BadInputException("Ambient space must be known for unit vector " + type_string + "!");
                         }
 
-                        vector<vector<Number> > e_i = vector<vector<Number> >(1, vector<Number>(nr_columns, 0));
+                        Matrix<Number> e_i = Matrix<Number>(vector<Number>(nr_columns, 0));
                         if (pos < 1 || pos > static_cast<long>(e_i[0].size())) {
                             throw BadInputException("Error while reading " + type_string + " as a unit_vector " + toString(pos) +
                                                     "!");
@@ -852,7 +1039,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         if (!success) {
                             throw BadInputException("Error while reading " + type_string + " as a sparse vector!");
                         }
-                        save_matrix(input_map, input_type, vector<vector<Number> >(1, sparse_vec));
+                        save_matrix(input_map, input_type, Matrix<Number>(sparse_vec));
                         continue;
                     }
 
@@ -867,7 +1054,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         if (!success || (long)formatted_vec.size() != nr_columns) {
                             throw BadInputException("Error while reading " + type_string + " as a formatted vector!");
                         }
-                        save_matrix(input_map, input_type, vector<vector<Number> >(1, formatted_vec));
+                        save_matrix(input_map, input_type, Matrix<Number>(formatted_vec));
                         continue;
                     }  // end formatted vector
                 }
@@ -875,7 +1062,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     in >> std::ws;
                     c = in.peek();
 
-                    if (c != '[' && !std::isdigit(c)) {  // must be transpose
+                    if (c != '[' && c != 'u' && !std::isdigit(c)) {  // must be transpose
                         string transpose_str;
                         in >> transpose_str;
                         if (transpose_str != "transpose") {
@@ -886,12 +1073,12 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         c = in.peek();
                     }
                     if (c == '[') {  // it is a formatted matrix
-                        vector<vector<Number> > formatted_mat;
+                        Matrix<Number> formatted_mat;
                         bool success = read_formatted_matrix(in, formatted_mat, transpose);
                         if (!success) {
                             throw BadInputException("Error while reading formatted matrix " + type_string + "!");
                         }
-                        if (formatted_mat.size() == 0) {  // empty matrix
+                        if (formatted_mat.nr_of_rows() == 0) {  // empty matrix
                             input_type = to_type(type_string);
                             save_empty_matrix(input_map, input_type);
                             continue;
@@ -908,7 +1095,27 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
 
                         save_matrix(input_map, input_type, formatted_mat);
                         continue;
-                    }  // only plain matrix left
+                    }
+                    if (c == 'u') {  // must be unit matrix
+                        string unit_test;
+                        in >> unit_test;
+                        if (unit_test != "unit_matrix") {
+                            throw BadInputException("Error while reading " + type_string + ": unit matrix expected!");
+                        }
+                        if (!dim_known) {
+                            throw BadInputException("Dimension must be known for unit matrix!");
+                        }
+                        nr_columns = dim + type_nr_columns_correction(input_type);
+                        Matrix<Number> unit_mat(nr_columns,nr_columns);
+                        nr_columns = dim + type_nr_columns_correction(input_type);
+                        for (long i = 0; i < nr_columns; ++i) {
+                            unit_mat[i][i] = 1;
+                        }
+                        save_matrix(input_map, input_type, unit_mat);
+                        continue;
+                    }
+
+                    // only plain matrix left
 
                     in >> nr_rows_or_columns;      // is number of columns if transposed
                     nr_rows = nr_rows_or_columns;  // most of the time
@@ -931,10 +1138,12 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     continue;
                 }
 
-                vector<vector<Number> > M(nr_rows);
+                Matrix<Number> M(nr_rows,nr_columns);
+                bool dense_matrix = true;
                 in >> std::ws;
                 c = in.peek();
                 if (c == 's') {  // must be sparse
+                    dense_matrix = false;
                     string sparse_test;
                     in >> sparse_test;
                     if (sparse_test != "sparse") {
@@ -947,7 +1156,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         }
                     }
                 }
-                else {  // dense matrix
+                if (dense_matrix) {  // dense matrix
                     for (i = 0; i < nr_rows; i++) {
                         M[i].resize(nr_columns);
                         for (j = 0; j < nr_columns; j++) {
@@ -957,7 +1166,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     }
                 }
                 if (transpose)
-                    M = transpose_mat(M);
+                    M = M.transpose();
                 save_matrix(input_map, input_type, M);
             }
             if (in.fail()) {
@@ -976,7 +1185,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
             if ((nr_rows < 0) || (nr_columns < 0)) {
                 throw BadInputException("Error while reading a " + toString(nr_rows) + "x" + toString(nr_columns) + " matrix !");
             }
-            vector<vector<Number> > M(nr_rows, vector<Number>(nr_columns));
+            Matrix<Number> M(nr_rows, nr_columns);
             for (i = 0; i < nr_rows; i++) {
                 for (j = 0; j < nr_columns; j++) {
                     read_number(in, M[i][j]);
@@ -998,18 +1207,24 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
     return input_map;
 }
 
-template map<Type::InputType, vector<vector<mpq_class> > > readNormalizInput(istream& in,
-                                                                 OptionsHandler& options,
-                                                                 map<NumParam::Param, long>& num_param_input,
-                                                                 string& polynomial,
-                                                                 renf_class& number_field);
+template InputMap<mpq_class> readNormalizInput(istream& in,
+                                                                             OptionsHandler& options,
+                                                                             map<NumParam::Param, long>& num_param_input,
+                                                                             string& polynomial,
+                                                                             renf_class_shared& number_field);
 
 #ifdef ENFNORMALIZ
-template map<Type::InputType, vector<vector<renf_elem_class> > > readNormalizInput(istream& in,
-                                                                 OptionsHandler& options,
-                                                                 map<NumParam::Param, long>& num_param_input,
-                                                                 string& polynomial,
-                                                                 renf_class& number_field);
+template InputMap<renf_elem_class> readNormalizInput(istream& in,
+                                                                                   OptionsHandler& options,
+                                                                                   map<NumParam::Param, long>& num_param_input,
+                                                                                   string& polynomial,
+                                                                                   renf_class_shared& number_field);
 #endif
 
-} // namespace
+#ifndef NMZ_MIC_OFFLOAD  // offload with long is not supported
+template Matrix<long> readMatrix(const string project);
+#endif  // NMZ_MIC_OFFLOAD
+template Matrix<long long> readMatrix(const string project);
+template Matrix<mpz_class> readMatrix(const string project);
+
+}  // namespace libnormaliz
